# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from .....olvid.daemon.admin.v1 import backup_admin_pb2 as olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2
from .....olvid.daemon.admin.v1 import client_key_admin_pb2 as olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2
from .....olvid.daemon.admin.v1 import identity_admin_pb2 as olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2


class ClientKeyAdminServiceStub(object):
    """ClientKey
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ClientKeyList = channel.unary_stream(
                '/olvid.daemon.services.v1.ClientKeyAdminService/ClientKeyList',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyListRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyListResponse.FromString,
                _registered_method=True)
        self.ClientKeyGet = channel.unary_unary(
                '/olvid.daemon.services.v1.ClientKeyAdminService/ClientKeyGet',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyGetRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyGetResponse.FromString,
                _registered_method=True)
        self.ClientKeyNew = channel.unary_unary(
                '/olvid.daemon.services.v1.ClientKeyAdminService/ClientKeyNew',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyNewRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyNewResponse.FromString,
                _registered_method=True)
        self.ClientKeyDelete = channel.unary_unary(
                '/olvid.daemon.services.v1.ClientKeyAdminService/ClientKeyDelete',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyDeleteRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyDeleteResponse.FromString,
                _registered_method=True)


class ClientKeyAdminServiceServicer(object):
    """ClientKey
    """

    def ClientKeyList(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClientKeyGet(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClientKeyNew(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClientKeyDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ClientKeyAdminServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ClientKeyList': grpc.unary_stream_rpc_method_handler(
                    servicer.ClientKeyList,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyListRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyListResponse.SerializeToString,
            ),
            'ClientKeyGet': grpc.unary_unary_rpc_method_handler(
                    servicer.ClientKeyGet,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyGetRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyGetResponse.SerializeToString,
            ),
            'ClientKeyNew': grpc.unary_unary_rpc_method_handler(
                    servicer.ClientKeyNew,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyNewRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyNewResponse.SerializeToString,
            ),
            'ClientKeyDelete': grpc.unary_unary_rpc_method_handler(
                    servicer.ClientKeyDelete,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyDeleteRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyDeleteResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'olvid.daemon.services.v1.ClientKeyAdminService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('olvid.daemon.services.v1.ClientKeyAdminService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class ClientKeyAdminService(object):
    """ClientKey
    """

    @staticmethod
    def ClientKeyList(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/olvid.daemon.services.v1.ClientKeyAdminService/ClientKeyList',
            olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyListRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyListResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ClientKeyGet(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/olvid.daemon.services.v1.ClientKeyAdminService/ClientKeyGet',
            olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyGetRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyGetResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ClientKeyNew(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/olvid.daemon.services.v1.ClientKeyAdminService/ClientKeyNew',
            olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyNewRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyNewResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ClientKeyDelete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/olvid.daemon.services.v1.ClientKeyAdminService/ClientKeyDelete',
            olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyDeleteRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_client__key__admin__pb2.ClientKeyDeleteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)


class IdentityAdminServiceStub(object):
    """Identity
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.IdentityList = channel.unary_stream(
                '/olvid.daemon.services.v1.IdentityAdminService/IdentityList',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityListRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityListResponse.FromString,
                _registered_method=True)
        self.IdentityAdminGet = channel.unary_unary(
                '/olvid.daemon.services.v1.IdentityAdminService/IdentityAdminGet',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetResponse.FromString,
                _registered_method=True)
        self.IdentityAdminGetBytesIdentifier = channel.unary_unary(
                '/olvid.daemon.services.v1.IdentityAdminService/IdentityAdminGetBytesIdentifier',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetBytesIdentifierRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetBytesIdentifierResponse.FromString,
                _registered_method=True)
        self.IdentityAdminGetInvitationLink = channel.unary_unary(
                '/olvid.daemon.services.v1.IdentityAdminService/IdentityAdminGetInvitationLink',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetInvitationLinkRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetInvitationLinkResponse.FromString,
                _registered_method=True)
        self.IdentityAdminDownloadPhoto = channel.unary_unary(
                '/olvid.daemon.services.v1.IdentityAdminService/IdentityAdminDownloadPhoto',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminDownloadPhotoRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminDownloadPhotoResponse.FromString,
                _registered_method=True)
        self.IdentityDelete = channel.unary_unary(
                '/olvid.daemon.services.v1.IdentityAdminService/IdentityDelete',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityDeleteRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityDeleteResponse.FromString,
                _registered_method=True)
        self.IdentityNew = channel.unary_unary(
                '/olvid.daemon.services.v1.IdentityAdminService/IdentityNew',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityNewRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityNewResponse.FromString,
                _registered_method=True)
        self.IdentityKeycloakNew = channel.unary_unary(
                '/olvid.daemon.services.v1.IdentityAdminService/IdentityKeycloakNew',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityKeycloakNewRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityKeycloakNewResponse.FromString,
                _registered_method=True)


class IdentityAdminServiceServicer(object):
    """Identity
    """

    def IdentityList(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def IdentityAdminGet(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def IdentityAdminGetBytesIdentifier(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def IdentityAdminGetInvitationLink(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def IdentityAdminDownloadPhoto(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def IdentityDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def IdentityNew(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def IdentityKeycloakNew(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_IdentityAdminServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'IdentityList': grpc.unary_stream_rpc_method_handler(
                    servicer.IdentityList,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityListRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityListResponse.SerializeToString,
            ),
            'IdentityAdminGet': grpc.unary_unary_rpc_method_handler(
                    servicer.IdentityAdminGet,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetResponse.SerializeToString,
            ),
            'IdentityAdminGetBytesIdentifier': grpc.unary_unary_rpc_method_handler(
                    servicer.IdentityAdminGetBytesIdentifier,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetBytesIdentifierRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetBytesIdentifierResponse.SerializeToString,
            ),
            'IdentityAdminGetInvitationLink': grpc.unary_unary_rpc_method_handler(
                    servicer.IdentityAdminGetInvitationLink,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetInvitationLinkRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetInvitationLinkResponse.SerializeToString,
            ),
            'IdentityAdminDownloadPhoto': grpc.unary_unary_rpc_method_handler(
                    servicer.IdentityAdminDownloadPhoto,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminDownloadPhotoRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminDownloadPhotoResponse.SerializeToString,
            ),
            'IdentityDelete': grpc.unary_unary_rpc_method_handler(
                    servicer.IdentityDelete,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityDeleteRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityDeleteResponse.SerializeToString,
            ),
            'IdentityNew': grpc.unary_unary_rpc_method_handler(
                    servicer.IdentityNew,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityNewRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityNewResponse.SerializeToString,
            ),
            'IdentityKeycloakNew': grpc.unary_unary_rpc_method_handler(
                    servicer.IdentityKeycloakNew,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityKeycloakNewRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityKeycloakNewResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'olvid.daemon.services.v1.IdentityAdminService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('olvid.daemon.services.v1.IdentityAdminService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class IdentityAdminService(object):
    """Identity
    """

    @staticmethod
    def IdentityList(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/olvid.daemon.services.v1.IdentityAdminService/IdentityList',
            olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityListRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityListResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def IdentityAdminGet(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/olvid.daemon.services.v1.IdentityAdminService/IdentityAdminGet',
            olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def IdentityAdminGetBytesIdentifier(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/olvid.daemon.services.v1.IdentityAdminService/IdentityAdminGetBytesIdentifier',
            olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetBytesIdentifierRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetBytesIdentifierResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def IdentityAdminGetInvitationLink(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/olvid.daemon.services.v1.IdentityAdminService/IdentityAdminGetInvitationLink',
            olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetInvitationLinkRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminGetInvitationLinkResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def IdentityAdminDownloadPhoto(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/olvid.daemon.services.v1.IdentityAdminService/IdentityAdminDownloadPhoto',
            olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminDownloadPhotoRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityAdminDownloadPhotoResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def IdentityDelete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/olvid.daemon.services.v1.IdentityAdminService/IdentityDelete',
            olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityDeleteRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityDeleteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def IdentityNew(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/olvid.daemon.services.v1.IdentityAdminService/IdentityNew',
            olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityNewRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityNewResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def IdentityKeycloakNew(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/olvid.daemon.services.v1.IdentityAdminService/IdentityKeycloakNew',
            olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityKeycloakNewRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_identity__admin__pb2.IdentityKeycloakNewResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)


class BackupAdminServiceStub(object):
    """Backup
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.BackupKeyGet = channel.unary_unary(
                '/olvid.daemon.services.v1.BackupAdminService/BackupKeyGet',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupKeyGetRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupKeyGetResponse.FromString,
                _registered_method=True)
        self.BackupKeyRenew = channel.unary_unary(
                '/olvid.daemon.services.v1.BackupAdminService/BackupKeyRenew',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupKeyRenewRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupKeyRenewResponse.FromString,
                _registered_method=True)
        self.BackupGet = channel.unary_unary(
                '/olvid.daemon.services.v1.BackupAdminService/BackupGet',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupGetRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupGetResponse.FromString,
                _registered_method=True)
        self.BackupNow = channel.unary_unary(
                '/olvid.daemon.services.v1.BackupAdminService/BackupNow',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupNowRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupNowResponse.FromString,
                _registered_method=True)
        self.BackupRestoreDaemon = channel.unary_unary(
                '/olvid.daemon.services.v1.BackupAdminService/BackupRestoreDaemon',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreDaemonRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreDaemonResponse.FromString,
                _registered_method=True)
        self.BackupRestoreAdminBackup = channel.unary_unary(
                '/olvid.daemon.services.v1.BackupAdminService/BackupRestoreAdminBackup',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreAdminBackupRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreAdminBackupResponse.FromString,
                _registered_method=True)
        self.BackupRestoreProfileSnapshot = channel.unary_unary(
                '/olvid.daemon.services.v1.BackupAdminService/BackupRestoreProfileSnapshot',
                request_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreProfileSnapshotRequest.SerializeToString,
                response_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreProfileSnapshotResponse.FromString,
                _registered_method=True)


class BackupAdminServiceServicer(object):
    """Backup
    """

    def BackupKeyGet(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BackupKeyRenew(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BackupGet(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BackupNow(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BackupRestoreDaemon(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BackupRestoreAdminBackup(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BackupRestoreProfileSnapshot(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_BackupAdminServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'BackupKeyGet': grpc.unary_unary_rpc_method_handler(
                    servicer.BackupKeyGet,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupKeyGetRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupKeyGetResponse.SerializeToString,
            ),
            'BackupKeyRenew': grpc.unary_unary_rpc_method_handler(
                    servicer.BackupKeyRenew,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupKeyRenewRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupKeyRenewResponse.SerializeToString,
            ),
            'BackupGet': grpc.unary_unary_rpc_method_handler(
                    servicer.BackupGet,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupGetRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupGetResponse.SerializeToString,
            ),
            'BackupNow': grpc.unary_unary_rpc_method_handler(
                    servicer.BackupNow,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupNowRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupNowResponse.SerializeToString,
            ),
            'BackupRestoreDaemon': grpc.unary_unary_rpc_method_handler(
                    servicer.BackupRestoreDaemon,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreDaemonRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreDaemonResponse.SerializeToString,
            ),
            'BackupRestoreAdminBackup': grpc.unary_unary_rpc_method_handler(
                    servicer.BackupRestoreAdminBackup,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreAdminBackupRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreAdminBackupResponse.SerializeToString,
            ),
            'BackupRestoreProfileSnapshot': grpc.unary_unary_rpc_method_handler(
                    servicer.BackupRestoreProfileSnapshot,
                    request_deserializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreProfileSnapshotRequest.FromString,
                    response_serializer=olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreProfileSnapshotResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'olvid.daemon.services.v1.BackupAdminService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('olvid.daemon.services.v1.BackupAdminService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class BackupAdminService(object):
    """Backup
    """

    @staticmethod
    def BackupKeyGet(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/olvid.daemon.services.v1.BackupAdminService/BackupKeyGet',
            olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupKeyGetRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupKeyGetResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def BackupKeyRenew(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/olvid.daemon.services.v1.BackupAdminService/BackupKeyRenew',
            olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupKeyRenewRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupKeyRenewResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def BackupGet(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/olvid.daemon.services.v1.BackupAdminService/BackupGet',
            olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupGetRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupGetResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def BackupNow(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/olvid.daemon.services.v1.BackupAdminService/BackupNow',
            olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupNowRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupNowResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def BackupRestoreDaemon(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/olvid.daemon.services.v1.BackupAdminService/BackupRestoreDaemon',
            olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreDaemonRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreDaemonResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def BackupRestoreAdminBackup(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/olvid.daemon.services.v1.BackupAdminService/BackupRestoreAdminBackup',
            olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreAdminBackupRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreAdminBackupResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def BackupRestoreProfileSnapshot(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/olvid.daemon.services.v1.BackupAdminService/BackupRestoreProfileSnapshot',
            olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreProfileSnapshotRequest.SerializeToString,
            olvid_dot_daemon_dot_admin_dot_v1_dot_backup__admin__pb2.BackupRestoreProfileSnapshotResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



import asyncio
import base64
import logging
import random
import string
from os import remove

import grpc.aio

from ClientHolder import ClientHolder, ClientWrapper
from olvid import datatypes
import ressources


# noinspection PyProtectedMember
async def create_identities(client_holder: ClientHolder, number_of_identities: int):
	#####
	# Create identities
	#####
	random_prefix = "".join(random.choice(string.ascii_letters) for _ in range(5))
	for i in range(number_of_identities):
		first_name = f"TestIdentity-{random_prefix}"
		last_name = str(i)
		position = "TO DEL"
		company = "Earth"
		identity = await client_holder.create_identity(first_name=first_name, last_name=last_name, position=position, company=company)
		expected_identity = datatypes.Identity(details=datatypes.IdentityDetails(first_name=first_name, last_name=last_name, position=position, company=company), display_name=f"{first_name} {last_name} ({position} @ {company})", keycloak_managed=False)
		assert identity._test_assertion(expected_identity)

	#####
	# Show created identities
	#####
	for client in client_holder.clients:
		logging.info(f"{client.identity.id}: {client.identity.details.first_name} {client.identity.details.last_name}")

	#####
	# Check admin identity get commands
	#####
	for client in client_holder.clients:
		assert client.identity == await client_holder.admin_client.admin_identity_admin_get(client.identity.id)
		assert await client.identity_get() == await client_holder.admin_client.admin_identity_admin_get(client.identity.id)


async def set_and_unset_identity_photo(client: ClientWrapper):
	# file that does not exist
	try:
		await client.identity_set_photo_file("this_file_does_not_exists.txt")
		raise Exception(f"Do not detected invalid file")
	except IOError:
		pass
	except Exception as e:
		print(type(e))
		raise Exception(f"Invalid exception raised: {e}")
	# check image is set (wait for DbCache to be up-to-date ...)
	await asyncio.sleep(0.1)
	assert not (await client.identity_get()).has_a_photo

	# valid filename, invalid payload
	try:
		with open("invalid_photo.png", "w") as fd:
			fd.write("This is not a photo !" * 20)
		await client.identity_set_photo_file("invalid_photo.png")
		raise Exception(f"Daemon accepted an invalid format image")
	except grpc.aio.AioRpcError:
		pass
	except Exception as e:
		raise Exception(f"Invalid exception raised: {e}")
	finally:
		remove("invalid_photo.png")
	# check image is set (wait for DbCache to be up-to-date ...)
	await asyncio.sleep(0.1)
	assert not (await client.identity_get()).has_a_photo

	# valid image
	with open("image.png", "wb") as fd:
		fd.write(base64.b64decode(ressources.PNG_IMAGE_AS_B64))
	await client.identity_set_photo_file("image.png")

	# valid image as bytes
	await client.identity_set_photo(filename="image.png", payload=base64.b64decode(ressources.PNG_IMAGE_AS_B64))

	await asyncio.sleep(0.1)  # added to be sure new photo is set in database
	# download image and check it's valid
	downloaded_image: bytes = await client.identity_download_photo()
	# downloaded image len is hardcoded for this given image, after daemon re-compressed it
	assert downloaded_image and len(downloaded_image) > 0, f"downloaded identity photo is invalid ({len(downloaded_image)})"

	# check image is set (wait for db to be up-to-date ...)
	# TODO add and wait for identity_photo_updated notification
	c = 0
	while not (await client.identity_get()).has_a_photo:
		await asyncio.sleep(0.1)
		c += 1
		if c == 100:
			assert (await client.identity_get()).has_a_photo

	# unset image
	await client.identity_remove_photo()

	# delete file on disk
	remove("image.png")


async def admin_test_get_identifier(client_holder: ClientHolder):
	async for identity in client_holder.admin_client.admin_identity_list():
		identifier_1: bytes = await client_holder.admin_client.admin_identity_admin_get_bytes_identifier(identity_id=identity.id)
		identifier_2: bytes = await client_holder.admin_client.admin_identity_admin_get_bytes_identifier(identity_id=identity.id)
		assert identifier_1, "identity identifier is empty"
		assert identifier_1 == identifier_2

async def admin_test_invitation_link(client_holder: ClientHolder):
	async for identity in client_holder.admin_client.admin_identity_list():
		link: str = await client_holder.admin_client.admin_identity_admin_get_invitation_link(identity_id=identity.id)
		assert link, "empty identity invitation link"
		assert link.startswith("https://invitation.olvid.io/#"), f"Invalid identity invitation link format: {link}"
		assert len(link.removeprefix("https://invitation.olvid.io/#")) > 0, f"Invalid identity invitation link payload: {link}"

async def test_get_identifier(c1: ClientWrapper):
	identifier_1: bytes = await c1.identity_get_bytes_identifier()
	identifier_2: bytes = await c1.identity_get_bytes_identifier()
	assert identifier_1, "identity identifier is empty"
	assert identifier_1 == identifier_2

async def test_invitation_link(c1: ClientWrapper):
	link: str = await c1.identity_get_invitation_link()
	assert link, "empty contact invitation link"
	assert link.startswith("https://invitation.olvid.io/#"), "Invalid contact invitation link format"
	assert len(link.removeprefix("https://invitation.olvid.io/#")) > 0, f"Invalid identity invitation link payload: {link}"


async def test_identity(client_holder: ClientHolder, number_of_identities: int):
	await create_identities(client_holder, number_of_identities)

	# admin identity tests
	await admin_test_get_identifier(client_holder)
	await admin_test_invitation_link(client_holder)

	for client in client_holder.clients:
		await set_and_unset_identity_photo(client)
		await test_get_identifier(client)
		await test_invitation_link(client)



from __future__ import annotations
from typing import TYPE_CHECKING

import re
import inspect

from . import GenericNotificationListener
from .ListenersImplementation import MessageReceivedListener
from ..datatypes import datatypes

if TYPE_CHECKING:
	from typing import Callable, Optional, Coroutine
	from .. import datatypes


class Command(MessageReceivedListener):
	"""
	Command: a specific MessageReceivedListener associated with a regexp that will determine if it's called or not.

	The most convenient way to add a command is to use the `OlvidClient.command` decorator, like this command are automatically
	added to an OlvidClient instance.
	Examples of possible way to add commands to a bot:

	```
	class Bot(OlvidClient):
		# use decorator in you bot class declaration
		@OlvidClient.command(regexp_filter="^!help")
		async def help_cmd(self, message: datatypes.Message):
			await message.reply("help message")

	bot = OlvidClient()

	# use decorator to to an existing bot instance
	@bot.command(regexp_filter="!second")
	async def second_command(message: datatypes.Message):
		print("Second command")

	# use add_listener method
	bot.add_listener(Command(regexp_filter="^!cmd", handler=lambda message: print(message)))
	```

	Command attributes:
	- regexp_filter (mandatory): the regexp that will determine if this command is called or not
	- ignore_case: bool (optional) (default= True): regexp ignore case
	- name: str (optional): can be used to create help messages
	- usage: str (optional): can be used to create help messages
	"""
	def __init__(self, regexp_filter: str, handler: Callable[[datatypes.Message], Optional[Coroutine]], name: str = None, usage: str = None, ignore_case: bool = True):
		# treat ignore_case and regexp_filter before subscribing to notifications
		self._regexp_filter: str = regexp_filter
		self._ignore_case = ignore_case
		if ignore_case and not self._regexp_filter.startswith("(?i)"):
			self._regexp_filter = "(?i)" + self._regexp_filter

		super().__init__(handler=self._handler_wrapper(handler), filter=datatypes.MessageFilter(body_search=self.regexp_filter))

		self._original_handler: Callable[[datatypes.Message], Optional[Coroutine]] = handler
		self._name = name if name is not None else regexp_filter
		self._usage: Optional[str] = usage

	def match(self, message: datatypes.Message) -> bool:
		return self.match_str(message.body)

	def match_str(self, message_body: str) -> bool:
		return re.match(self._regexp_filter, message_body, re.IGNORECASE if self._ignore_case else 0) is not None

	def _handler_wrapper(self, original_handler) -> Callable[[datatypes.Message], Optional[Coroutine]]:
		async def wrapped_command_handler(message: datatypes.Message):
			signature = inspect.signature(original_handler)
			if len(signature.parameters) == 1:
				ret = original_handler(message)
			else:
				match_body = re.match(self._regexp_filter, message.body, re.IGNORECASE if self._ignore_case else 0)
				ret = original_handler(message, match_body.group())
			if inspect.iscoroutine(ret):
				await ret
		return wrapped_command_handler

	@property
	def regexp_filter(self) -> str:
		return self._regexp_filter

	@property
	def name(self) -> str:
		return self._name

	@property
	def usage(self) -> str:
		return self._usage

	@staticmethod
	def get_default_regexp(name: str):
		return f"^!{name}"

	def __str__(self):
		return f"{self._name}{': ' + self.usage if self.usage else ''}"


#####
# Decorator section
#####
# Abstract class, contains __call__ method to create a decorator to add commands to a bot.
# This class have two implementations:
# - ClassCommandDecorator: embed it at class level, to annotate methods in class definition.
# it will store method names that you want to add as commands. Then on bot instance creation it will
# retrieve all these methods (using their names), create appropriated Command instances and add it as listeners.
# We need to use method instead of functions that's why we store function names and then resolve methods using getattr when bot is instantiated.
# - InstanceCommandDecorator: this is supposed to override the class attribute containing ClassCommandDecorator when a bot
# is instantiated. On decorator use it will create a new Command and add it to bot listeners.
# this is valid decorator usage: @command, @command(), @command("regexp"), @command("regexp", "usage"),
#   @command(regexp="regexp", usage="usage"), @command(usage="usage")
class CommandDecorator:
	def __call__(self, *args, regexp_filter: str = None, usage: str = None, name: str = None, ignore_case: bool = None):
		## handle @command call: store command using function name as command name, and return function
		if args and callable(args[0]):
			function = args[0]
			regexp_filter = Command.get_default_regexp(function.__name__)
			self._add_command(function=function, regexp_filter=regexp_filter)
			return function

		####
		# handle @command() call: expect to return a decorator, so create a decorator that will create the command using initial wrapper optional arguments
		####
		# if there are no named arguments try to determine arguments
		if regexp_filter is None and usage is None and name is None:
			if len(args) == 3:
				regexp_filter = args[0]
				usage = args[1]
				name = args[2]
			elif len(args) == 2:
				regexp_filter = args[0]
				usage = args[1]
			elif len(args) == 1:
				regexp_filter = args[0]
			elif len(args) == 0:
				regexp_filter = ""
			else:
				raise TypeError(f"invalid number of parameters: {len(args)}")
		else:
			regexp_filter = regexp_filter

		# check argument types
		if regexp_filter is None or type(regexp_filter) is not str:
			raise ValueError("Command: regexp_filter is required and must be a string")
		if usage is not None and type(usage) is not str:
			raise ValueError("Command: usage must be a string")
		if name is not None and type(name) is not str:
			raise ValueError("Command: name must be a string")
		if ignore_case is not None and type(ignore_case) is not bool:
			raise ValueError("Command: ignore_case must be a boolean")

		# handle @command() call: expect to return a decorator, so create a decorator that will create the command using initial wrapper optional arguments
		def decorator(func):
			# add command (handle @command() with no arguments)
			if not regexp_filter:
				self._add_command(function=func, regexp_filter=Command.get_default_regexp(func.__name__), usage=usage, name=name, ignore_case=ignore_case)
			else:
				self._add_command(function=func, regexp_filter=regexp_filter, usage=usage, name=name, ignore_case=ignore_case)
			return func
		return decorator

	# method to override by implementations
	def _add_command(self, function: callable, regexp_filter: str, usage: str = None, name: str = None, ignore_case: bool = None):
		pass

	# method to override by implementations
	def get_commands(self, instance: "CommandHolder") -> list[Command]:
		pass


class ClassCommandDecorator(CommandDecorator):
	def __init__(self):
		# We use a dict to store commands by class name. This allows computing commands inherited from parents.
		# We store every command by name to allow command overriding.
		# For every command, we store constructor params because we will only create Command on Bot creation to use instance
		# methods as handlers.
		# Data structure: dict: bot class_name -> dict2
		# dict2: regexp_filter -> (class_method_name, usage, name)
		self._command_elements_by_name_by_class_name: dict[str, dict[str, tuple[str, str, str, bool]]] = {}

	# add command to the global OlvidClient command holder
	def _add_command(self, function: callable, regexp_filter: str, usage: str = None, name: str = None, ignore_case: bool = None):
		class_name = function.__qualname__.removesuffix(f".{function.__name__}")

		if not self._command_elements_by_name_by_class_name.get(class_name):
			self._command_elements_by_name_by_class_name[class_name] = {}
		self._command_elements_by_name_by_class_name[class_name][regexp_filter] = (function.__name__, usage, name, ignore_case)

	def get_commands(self, instance: "CommandHolder") -> list[Command]:
		commands_dict: dict[str, Command] = {}
		for class_ in reversed(instance.__class__.mro()):
			if class_ == object:
				continue
			if self._command_elements_by_name_by_class_name.get(class_.__name__):
				for regexp, (command_handler_name, usage, name, ignore_case) in self._command_elements_by_name_by_class_name.get(class_.__name__).items():
					command_constructor_params = {
						"regexp_filter": regexp,
						"handler": getattr(instance, command_handler_name),
					}
					if name is not None:
						command_constructor_params["name"] = name
					if usage is not None:
						command_constructor_params["usage"] = usage
					if ignore_case is not None:
						command_constructor_params["ignore_case"] = ignore_case
					commands_dict[regexp] = Command(**command_constructor_params)
		return list(commands_dict.values())


#####
# Abstract class to implement to have a command decorator.
# It will add a class attribute for command decorator. This will store the commands for this class and its child classes.
# When you instantiate this class it will override previous command attribute. The new InstanceCommandDecorator determine
# commands associated to this instance browsing ClassCommandDecorator registered commands using instance __bases__.
# When new decorator is called it will directly create Command and add it to bot listener.
#####
class CommandHolder:
	command: CommandDecorator = ClassCommandDecorator()

	def __init__(self):
		for c in self.__class__.command.get_commands(self):
			self.add_listener(c)
		self.command = InstanceCommandDecorator(self)

	def add_listener(self, listener: GenericNotificationListener):
		raise NotImplementedError(f"{self.__class__.__name__}: CommandHolder: add_listener not implemented")

	def remove_listener(self, listener: GenericNotificationListener):
		raise NotImplementedError(f"{self.__class__.__name__}: CommandHolder: remove_listener not implemented")


class InstanceCommandDecorator(CommandDecorator):
	def __init__(self, instance: CommandHolder):
		super(CommandDecorator, self).__init__()
		# retrieve commands from ClassLevel command decorator
		self._holder_instance: CommandHolder = instance
		self._command_list: list[Command] = self._holder_instance.__class__.command.get_commands(
			self._holder_instance)

	# method to override by implementations
	def _add_command(self, function: callable, regexp_filter: str, usage: str = None, name: str = None, ignore_case: bool = None):
		command_constructor_params = {
			"regexp_filter": regexp_filter,
			"handler": function,
		}
		if name is not None:
			command_constructor_params["name"] = name
		if usage is not None:
			command_constructor_params["usage"] = usage
		if ignore_case is not None:
			command_constructor_params["ignore_case"] = ignore_case
		new_command = Command(**command_constructor_params)
		# check if a command is already registered with this regexp_filter
		filtered_commands: list[Command] = list(filter(lambda c: c.regexp_filter == regexp_filter, self._command_list))

		if not filtered_commands:
			self._command_list.append(new_command)
			self._holder_instance.add_listener(new_command)
		# overriding existing command
		else:
			previous_command = filtered_commands[0]
			self._command_list.remove(previous_command)
			self._holder_instance.remove_listener(previous_command)
			self._holder_instance.add_listener(new_command)

	def get_commands(self, instance: "CommandHolder") -> list[Command]:
		return self._command_list



import json
from .. import datatypes

try:
	import aiohttp
	_has_aio_http = True
except ImportError:
	aiohttp = None
	_has_aio_http = False

# this class must be used with a keycloak managed identity and bot must have a valid role on keycloak
# !! required aiohttp module !!
class KeycloakAdminApiClient:
	def __init__(self, keycloak_credentials: datatypes.KeycloakApiCredentials):
		"""requires aiohttp to be installed"""
		if not _has_aio_http:
			raise ImportError('aiohttp must be installed to use KeycloakAdminApiClient, use pip3 install \'olvid-bot[keycloak]\'')

		self.credentials: datatypes.KeycloakApiCredentials = keycloak_credentials

	async def get_your_realm_name(self):
		# parse server url to extract realm name
		return [s for s in self.credentials.server_url.split("/") if s][-1]

	# return a list of realms: {"realmList":[{"name": str, "admin": bool, "olvid": bool, [...]}]}
	async def list_realms(self, only_olvid_realms: bool = False, only_admin_realms: bool = False) -> list[dict]:
		payload: dict = {"q": 3}
		response = await self._admin_api_request(payload)
		realmList = response.get("data").get("realmList")
		if only_olvid_realms:
			return [realm for realm in realmList if realm.get("olvid")]
		elif only_admin_realms:
			return [realm for realm in realmList if realm.get("admin")]
		else:
			return realmList

	# return new bot configuration link
	# possible roles: "admin", "editor", "viewer", "none"
	async def create_bot(self, realmName: str, username: str, identity_details: datatypes.IdentityDetails, role: str = "none") -> str:
		payload: dict = {
		  "q": 19,
		  "realmName": realmName,
		  "username": username,
		  "firstname": identity_details.first_name,
		  "lastname": identity_details.last_name,
		  "position": identity_details.position,
		  "company": identity_details.company,
		  "role": role
		}
		return await self._admin_api_request(payload)

	# return {"usersList": [{"id":str,"username":str,"email":str,"firstname":str,"lastname":str,"position":str,"company":str,[...]}]}
	async def get_user_by_username(self, realm_name: str, username: str) -> dict:
		payload: dict = {
		  "q": 25,
		  "realmName": realm_name,
		  "usersUsernameList": [username],
		}
		return (await self._admin_api_request(payload)).get("data")

	# return password
	async def create_user(self, realmName: str, username: str, password: str, identity_details: datatypes.IdentityDetails, email: str = "") -> str:
		payload: dict = {
			"q": 24,
			"realmName": realmName,
			"isNoPassword": False,
			"user": {
				"email": email,
				"username": username,
				"firstname": identity_details.first_name,
				"lastname": identity_details.last_name,
				"position": identity_details.position,
				"company": identity_details.company,
				"password": password
			}
		}
		return (await self._admin_api_request(payload)).get("data").get("password")

	async def delete_user(self, realmName: str, username: str) -> str:
		payload: dict = {
			"q": 39,
			"realmName": realmName,
			"usersUsernameList": [username],
			"revocationType": 1
		}
		return (await self._admin_api_request(payload)).get("data").get("password")

	# return a realm dict: {"configLink":str,"writeAllowed":bool,"enableBotsManagement":bool,[...]}
	async def get_realm_settings(self, realm_name: str) -> dict:
		payload: dict = {"q": 43, "realmName": realm_name}
		return (await self._admin_api_request(payload)).get("data")

	# return a response dict: {"created":[{"link":str, "id":str, "tag":str, [...]}], "errors":[]}
	async def create_external_link(self, realm_name: str, tags: list[str]) -> dict:
		payload: dict = {"q": 58, "realmName": realm_name, "dataArray": tags}
		return (await self._admin_api_request(payload)).get("data")

	async def _admin_api_request(self, payload: dict):
		async with aiohttp.ClientSession() as session:
			headers = {
				"Content-Type": "application/json;charset=UTF-8",
				"direct-auth-username": self.credentials.username,
				"direct-auth-token": self.credentials.direct_auth_token,
			}
			async with session.post(url=self.credentials.server_url.rstrip("/") + "/olvid-rest/configuration", headers=headers, data=json.dumps(payload)) as resp:
				json_response = await resp.json()
				if type(json_response) is dict and json_response.get("error"):
					raise Exception(f"{json_response['message']} [code: {json_response.get('error')}]" if json_response.get("message") else str(json_response["error"]))
				return json_response



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: olvid/daemon/notification/v1/attachment_notifications.proto
# Protobuf Python Version: 6.33.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    33,
    2,
    '',
    'olvid/daemon/notification/v1/attachment_notifications.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .....olvid.daemon.datatypes.v1 import attachment_pb2 as olvid_dot_daemon_dot_datatypes_dot_v1_dot_attachment__pb2
from .....olvid.daemon.datatypes.v1 import message_pb2 as olvid_dot_daemon_dot_datatypes_dot_v1_dot_message__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n;olvid/daemon/notification/v1/attachment_notifications.proto\x12\x1colvid.daemon.notification.v1\x1a*olvid/daemon/datatypes/v1/attachment.proto\x1a\'olvid/daemon/datatypes/v1/message.proto\"\xa5\x01\n)SubscribeToAttachmentReceivedNotification\x12\x19\n\x05\x63ount\x18\x01 \x01(\x04H\x00R\x05\x63ount\x88\x01\x01\x12H\n\x06\x66ilter\x18\x02 \x01(\x0b\x32+.olvid.daemon.datatypes.v1.AttachmentFilterH\x01R\x06\x66ilter\x88\x01\x01\x42\x08\n\x06_countB\t\n\x07_filter\"g\n\x1e\x41ttachmentReceivedNotification\x12\x45\n\nattachment\x18\x01 \x01(\x0b\x32%.olvid.daemon.datatypes.v1.AttachmentR\nattachment\"\xbc\x02\n)SubscribeToAttachmentUploadedNotification\x12\x19\n\x05\x63ount\x18\x01 \x01(\x04H\x00R\x05\x63ount\x88\x01\x01\x12H\n\x06\x66ilter\x18\x02 \x01(\x0b\x32+.olvid.daemon.datatypes.v1.AttachmentFilterH\x01R\x06\x66ilter\x88\x01\x01\x12\x45\n\x0bmessage_ids\x18\x03 \x03(\x0b\x32$.olvid.daemon.datatypes.v1.MessageIdR\nmessageIds\x12N\n\x0e\x61ttachment_ids\x18\x04 \x03(\x0b\x32\'.olvid.daemon.datatypes.v1.AttachmentIdR\rattachmentIdsB\x08\n\x06_countB\t\n\x07_filter\"g\n\x1e\x41ttachmentUploadedNotification\x12\x45\n\nattachment\x18\x01 \x01(\x0b\x32%.olvid.daemon.datatypes.v1.AttachmentR\nattachmentB\xe4\x01\n com.olvid.daemon.notification.v1B\x1c\x41ttachmentNotificationsProtoP\x01Z\x0folvid.io/daemon\xa2\x02\x03ODN\xaa\x02\x1cOlvid.Daemon.Notification.V1\xca\x02\x1cOlvid\\Daemon\\Notification\\V1\xe2\x02(Olvid\\Daemon\\Notification\\V1\\GPBMetadata\xea\x02\x1fOlvid::Daemon::Notification::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'olvid.daemon.notification.v1.attachment_notifications_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n com.olvid.daemon.notification.v1B\034AttachmentNotificationsProtoP\001Z\017olvid.io/daemon\242\002\003ODN\252\002\034Olvid.Daemon.Notification.V1\312\002\034Olvid\\Daemon\\Notification\\V1\342\002(Olvid\\Daemon\\Notification\\V1\\GPBMetadata\352\002\037Olvid::Daemon::Notification::V1'
  _globals['_SUBSCRIBETOATTACHMENTRECEIVEDNOTIFICATION']._serialized_start=179
  _globals['_SUBSCRIBETOATTACHMENTRECEIVEDNOTIFICATION']._serialized_end=344
  _globals['_ATTACHMENTRECEIVEDNOTIFICATION']._serialized_start=346
  _globals['_ATTACHMENTRECEIVEDNOTIFICATION']._serialized_end=449
  _globals['_SUBSCRIBETOATTACHMENTUPLOADEDNOTIFICATION']._serialized_start=452
  _globals['_SUBSCRIBETOATTACHMENTUPLOADEDNOTIFICATION']._serialized_end=768
  _globals['_ATTACHMENTUPLOADEDNOTIFICATION']._serialized_start=770
  _globals['_ATTACHMENTUPLOADEDNOTIFICATION']._serialized_end=873
# @@protoc_insertion_point(module_scope)



import logging

from olvid import errors
from ClientHolder import ClientHolder

async def test_tool(client_holder: ClientHolder, fast_mode=False):
	logging.info(f"tools: ping")
	for c in client_holder.clients:
		await c.ping()

	# test with admin client
	try:
		await client_holder.admin_client.authentication_test()
	except errors.PermissionDeniedError as e:
		pass
	else:
		assert False, "authentication passed for non admin key"

	await client_holder.admin_client.authentication_admin_test()

	for c in client_holder.clients:
		# test with non admin client
		try:
			await c.authentication_admin_test()
		except errors.PermissionDeniedError as e:
			pass
		else:
			assert False, "authentication passed for non admin key"
		await c.authentication_test()



import asyncio

from olvid import datatypes, OlvidAdminClient, listeners


# noinspection PyProtectedMember
async def invitation_task(identity_1: datatypes.Identity, identity_2: datatypes.Identity):
	global count, total_count

	client_1: OlvidAdminClient = OlvidAdminClient(identity_id=identity_1.id)
	client_2: OlvidAdminClient = OlvidAdminClient(identity_id=identity_2.id)
	# check if identity is already a contact
	details_search: datatypes.IdentityDetails = identity_2.details._clone()
	details_search.first_name = f"^{details_search.first_name}$"
	details_search.last_name = f"^{details_search.last_name}$"
	details_search.position = f"^{details_search.position}$"
	details_search.company = f"^{details_search.company}$"
	if [c async for c in client_1.contact_list(filter=datatypes.ContactFilter(details_search=details_search))]:
		count += 1
		print(f"{count:3} / {total_count:3} skipped: {identity_1.id} -> {identity_2.id}")
		return

	print(f"Processing: {identity_1.id} -> {identity_2.id}")

	# send invitation
	invitation_1: datatypes.Invitation = await client_1.invitation_new(identity_2.invitation_url)

	# wait for invitation to arrive
	invitation_store_2: list[datatypes.Invitation] = []
	listener = listeners.InvitationReceivedListener(lambda i: invitation_store_2.append(i), filter=datatypes.InvitationFilter(display_name_search=f"{identity_1.details.first_name} {identity_1.details.last_name}".strip()), count=1)
	client_2.add_listener(listener)
	await client_2.wait_for_listeners_end()
	invitation_2: datatypes.Invitation = invitation_store_2[0]

	# accept invitation 2
	await client_2.invitation_accept(invitation_id=invitation_2.id)

	# wait for invitation to be ready to set sas
	invitation_store_1: list[datatypes.Invitation] = []
	listener_1 = listeners.InvitationUpdatedListener(handler=lambda i, p: invitation_store_1.append(i), invitation_ids=[invitation_1.id], filter=datatypes.InvitationFilter(status=datatypes.Invitation.Status.STATUS_INVITATION_WAIT_YOU_FOR_SAS_EXCHANGE), count=1)
	client_1.add_listener(listener_1)

	invitation_store_2: list[datatypes.Invitation] = []
	listener_2 = listeners.InvitationUpdatedListener(handler=lambda i, p: invitation_store_2.append(i), invitation_ids=[invitation_2.id], filter=datatypes.InvitationFilter(status=datatypes.Invitation.Status.STATUS_INVITATION_WAIT_YOU_FOR_SAS_EXCHANGE), count=1)
	client_2.add_listener(listener_2)

	await client_1.wait_for_listeners_end()
	await client_2.wait_for_listeners_end()

	invitation_1 = invitation_store_1[0]
	invitation_2 = invitation_store_2[0]

	# set sas
	await client_1.invitation_sas(invitation_id=invitation_1.id, sas=invitation_2.sas)
	await client_2.invitation_sas(invitation_id=invitation_2.id, sas=invitation_1.sas)

	count += 1
	print(f"{count:3} / {total_count:3}: finished: {identity_1.id} -> {identity_2.id}")


count: int = 0
total_count: int = 0


async def auto_invite(identity_id: int, admin_client: OlvidAdminClient, full: bool = False):
	global total_count, count
	identities: list[datatypes.Identity] = [i async for i in admin_client.admin_identity_list()]
	len_identities: int = len(identities)
	count = 0
	if full:
		total_count = round(len_identities * ((len_identities - 1) / 2))
	else:
		total_count = len_identities - 1

	# full mode
	if full:
		total_count = round(len_identities * ((len_identities - 1) / 2))
		for i in range(len(identities)):
			identity_1: datatypes.Identity = identities[i]
			tasks = []
			for j in range(i, len_identities):
				identity_2: datatypes.Identity = identities[j]
				if identity_1.id == identity_2.id:
					continue
				tasks.append(admin_client.add_background_task(invitation_task(identities[i], identities[j])))
			print(f"Queued {len(tasks)} tasks")
			await asyncio.gather(*tasks)
	else:
		current_identity: datatypes.Identity = await admin_client.admin_identity_admin_get(identity_id=identity_id)
		tasks = []
		for i in range(len(identities)):
			identity_1: datatypes.Identity = identities[i]
			if identity_1.id == current_identity.id:
				continue
			tasks.append(admin_client.add_background_task(invitation_task(current_identity, identities[i])))
		print(f"Queued {len(tasks)} tasks")
		await asyncio.gather(*tasks)




####
# WARNING: DO NOT EDIT: this code is automatically generated, see overlay_generator/generate_protobuf_overlay
####

from .types import NotificationMessageType, CommandRequestMessageType, CommandResponseMessageType
del types
